"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = __importStar(require("@oclif/command"));
const preview_1 = require("../../sdk/preview");
const output_1 = require("../../utils/output");
class Preview extends command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Preview);
            const message = yield preview_1.runPreviewRelease({
                cwd: process.cwd(),
                changelog: true,
                dryRun: flags['dry-run'],
                json: flags.json,
                progress: !flags.json,
                overrides: {
                    skipNpm: flags['skip-npm'],
                    buildNum: flags['build-num'],
                    trunk: flags.trunk,
                },
            });
            output_1.output(message, { json: flags.json });
        });
    }
}
exports.Preview = Preview;
Preview.flags = {
    trunk: command_1.flags.string({
        default: '',
        description: 'State which branch is trunk. Defaults to honuring the "base" branch setting in the GitHub repo settings.',
    }),
    ['build-num']: command_1.flags.integer({
        description: 'Force a build number. Should not be needed generally. For exceptional cases.',
        char: 'n',
    }),
    'dry-run': command_1.flags.boolean({
        default: false,
        description: 'output what the next version would be if released now',
        char: 'd',
    }),
    'skip-npm': command_1.flags.boolean({
        default: false,
        description: 'skip the step of publishing the package to npm',
    }),
    json: command_1.flags.boolean({
        default: false,
        description: 'format output as JSON',
        char: 'j',
    }),
};
