"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const nodefs = __importStar(require("fs"));
const isomorphic_git_1 = __importDefault(require("isomorphic-git"));
const node_1 = __importDefault(require("isomorphic-git/http/node"));
class Git2 {
    constructor(input) {
        var _a;
        this.dir = (_a = input === null || input === void 0 ? void 0 : input.cwd) !== null && _a !== void 0 ? _a : process.cwd();
        this.fs = nodefs;
        this.http = node_1.default;
    }
    /**
     * Get the name of the currently checked out branch.
     */
    getCurrentBranchName() {
        return isomorphic_git_1.default.currentBranch({ fs: this.fs, dir: this.dir });
    }
    /**
     * Check how the local branch is not in sync or is with the remote.
     * Ref: https://stackoverflow.com/questions/3258243/check-if-pull-needed-in-git
     */
    checkSyncStatus(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let remoteUrl = yield isomorphic_git_1.default.getConfig({
                fs: this.fs,
                dir: this.dir,
                path: 'remote.origin.url',
            });
            if (remoteUrl.startsWith('git@github.com:')) {
                remoteUrl = remoteUrl.replace('git@github.com:', 'https://github.com/');
            }
            if (!remoteUrl.endsWith('.git')) {
                remoteUrl = remoteUrl + '.git';
            }
            let remoteInfo;
            try {
                remoteInfo = yield isomorphic_git_1.default.getRemoteInfo({
                    http: this.http,
                    url: remoteUrl,
                });
            }
            catch (e) {
                throw new Error(`Failed to fetch remote info from ${remoteUrl} due to error:\n\n${e}`);
            }
            if (!remoteInfo.refs) {
                throw new Error('Could not fetch refs');
            }
            if (!remoteInfo.refs.heads) {
                throw new Error('Could not fetch ref heads');
            }
            if (!Object.keys(remoteInfo.refs.heads).find((remoteBranchName) => remoteBranchName === input.branchName)) {
                return 'remote_needs_branch';
            }
            const localBranchHeadSha = yield isomorphic_git_1.default.resolveRef({ fs: this.fs, dir: this.dir, ref: 'HEAD' });
            const remoteBranchHeadSha = remoteInfo.refs.heads[input.branchName];
            if (localBranchHeadSha === remoteBranchHeadSha) {
                return 'synced';
            }
            return 'not_synced';
            // todo https://github.com/isomorphic-git/isomorphic-git/issues/1110
        });
    }
}
function createGit(input) {
    return new Git2(input);
}
exports.createGit = createGit;
