"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const semver_1 = require("./semver");
/**
 * Publish a changelog. The given git tag should have already been pushed to the
 * remote. If the release is a preview then the github release will be made
 * against the pre-release identifier name. Otherwise the github release will be
 * made against the styled version.
 */
function publishChangelog(input) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const { octokit, release, repo: { owner, name: repo }, } = input;
        let res;
        try {
            if (semver_1.isStable(release.version)) {
                res = yield octokit.repos.createRelease({
                    owner,
                    repo,
                    prerelease: false,
                    tag_name: semver_1.renderStyledVersion(release.version),
                    draft: (_b = (_a = input.options) === null || _a === void 0 ? void 0 : _a.draft) !== null && _b !== void 0 ? _b : false,
                    body: input.body,
                });
                const existingPreviewRelease = yield maybeGetRelease({ octokit, owner, repo, tag: 'next' });
                if (existingPreviewRelease) {
                    res = yield octokit.repos.updateRelease({
                        owner,
                        repo,
                        release_id: existingPreviewRelease.data.id,
                        target_commitish: release.head.sha,
                        body: 'None since last stable.',
                    });
                }
            }
            else if (semver_1.isPreview(release.version)) {
                const v = release.version;
                const tag = v.preRelease.identifier;
                const existingPreviewRelease = yield maybeGetRelease({ octokit, owner, repo, tag });
                if (!existingPreviewRelease) {
                    res = yield octokit.repos.createRelease({
                        owner,
                        repo,
                        prerelease: true,
                        tag_name: v.preRelease.identifier,
                        draft: (_d = (_c = input.options) === null || _c === void 0 ? void 0 : _c.draft) !== null && _d !== void 0 ? _d : false,
                        body: input.body,
                    });
                }
                else {
                    res = yield octokit.repos.updateRelease({
                        owner,
                        repo,
                        release_id: existingPreviewRelease.data.id,
                        body: input.body,
                    });
                }
            }
            else {
                // Should never happen if used correctly.
                throw new Error(`WARNING: release notes are not supported for this kind of release: ${util_1.inspect(release)}`);
            }
        }
        catch (e) {
            throw new Error(`Failed to publish changelog\n\n${util_1.inspect(e)}`);
        }
        return res;
    });
}
exports.publishChangelog = publishChangelog;
function maybeGetRelease(input) {
    return __awaiter(this, void 0, void 0, function* () {
        let res = null;
        try {
            res = yield input.octokit.repos.getReleaseByTag({
                owner: input.owner,
                repo: input.repo,
                tag: input.tag,
            });
        }
        catch (error) {
            if (error.status !== 404) {
                throw new Error(`Failed to fetch releases for tag ${input.tag} on repo ${input.owner}/${input.repo}.\n\n${util_1.inspect(error)}`);
            }
        }
        return res;
    });
}
