"use strict";
/**
 * This module handles the concerns of publishing. It handles interaction with
 * git tagging, pushing to the git origin, the package registry, etc.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const promise_1 = __importDefault(require("simple-git/promise"));
const github_ci_environment_1 = require("./github-ci-environment");
const Pacman = __importStar(require("./pacman"));
const optionDefaults = {
    gitTagVPrefix: false,
    npm: true,
    gitTag: 'all',
};
/**
 * Run the publishing process.
 *
 * 1. Change package.json version field to be new version.
 * 2. npm publish --tag next.
 * 3. discard package.json change.
 * 4. git tag {newVer}.
 * 5. git tag next.
 * 6. git push --tags.
 *
 */
function publishPackage(input) {
    var _a;
    return __asyncGenerator(this, arguments, function* publishPackage_1() {
        const release = input.release;
        const opts = Object.assign(Object.assign({}, optionDefaults), input.options);
        if (opts.npm) {
            // publish to the npm registry
            //
            // If we are using a script runner then publish with that same tool. Otherwise
            // default to using npm. The reason we need to do this is that problems occur
            // when mixing tools. For example `yarn run ...` will lead to a spawn of `npm
            // publish` failing due to an authentication error.
            const pacman = yield __await(Pacman.create({ defualt: 'npm' }));
            yield __await(pacman.publish({ version: release.version, tag: release.distTag }));
            yield yield __await({ kind: 'package_published' }
            // todo parallel optimize?
            );
            // todo parallel optimize?
            if (release.extraDistTags) {
                for (const distTag of release.extraDistTags) {
                    yield __await(pacman.tag({ packageVersion: release.version, tagName: distTag }));
                    yield yield __await({ kind: 'extra_dist_tag_updated', distTag });
                }
            }
        }
        // While the fields of the package.json should not have changed, its
        // formatting, like indentation level, might have. We do not want to leave a
        // dirty working directoy on the user's system.
        //
        // TODO no invariant in system that checks that package.json was not modified
        // before beginning the publishing process. In other words we may be losing
        // user work here. This check should be in strict mode.
        const git = promise_1.default();
        yield __await(setupGitUsernameAndEmailOnCI(git));
        yield __await(git.checkout('package.json'));
        yield yield __await({ kind: 'package_json_reverted' }
        // Tag the git commit
        //
        );
        // Tag the git commit
        //
        const versionTag = opts.gitTagVPrefix ? 'v' + release.version : release.version;
        if (opts.gitTag === 'all' || opts.gitTag === 'just_version') {
            yield __await(git.addAnnotatedTag(versionTag, versionTag)
            // Avoid general git push tags otherwise we could run into trying to push e.g.
            // old `next` tag (dist-tags, forced later) that was since updated on remote
            // by CI––assuming user is doing a publish from their machine (maybe stable
            // for example).
            // Ref: https://stackoverflow.com/questions/23212452/how-to-only-push-a-specific-tag-to-remote
            );
            // Avoid general git push tags otherwise we could run into trying to push e.g.
            // old `next` tag (dist-tags, forced later) that was since updated on remote
            // by CI––assuming user is doing a publish from their machine (maybe stable
            // for example).
            // Ref: https://stackoverflow.com/questions/23212452/how-to-only-push-a-specific-tag-to-remote
            yield __await(git.raw(['push', 'origin', `refs/tags/${versionTag}`]));
            yield yield __await({ kind: 'version_git_tag_created' });
        }
        // Tag the git commit with the given dist tag names
        //
        if (opts.gitTag === 'all' || opts.gitTag === 'just_dist_tags') {
            // todo parallel optimize?
            const distTags = [release.distTag, ...((_a = release.extraDistTags) !== null && _a !== void 0 ? _a : [])];
            for (const distTag of distTags) {
                // dist tags are movable pointers. Except for init case it is expected to
                // exist in the git repo. So use force to move the tag.
                // https://stackoverflow.com/questions/8044583/how-can-i-move-a-tag-on-a-git-branch-to-a-different-commit
                // todo provide nice semantic descriptions for each dist tag
                yield __await(git.raw(['tag', '--force', '--message', distTag, distTag]));
                yield yield __await({ kind: 'extra_dist_tag_git_tag_created', distTag });
                yield __await(git.raw(['push', '--force', '--tags']));
                yield yield __await({ kind: 'extra_dist_tag_git_tag_pushed', distTag });
            }
        }
    });
}
exports.publishPackage = publishPackage;
// todo turn this into a check
/**
 * On CI set the local Git config user email and name if not already set on the
 * machine. If not CI this function is a no-op.
 *
 * @remarks
 *
 * It can happen that no user name or email is setup on a machine for git.
 * Certain git commands fail in that case like creating annotated tags. Ref:
 * https://stackoverflow.com/questions/11656761/git-please-tell-me-who-you-are-error.
 */
function setupGitUsernameAndEmailOnCI(git) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!github_ci_environment_1.isGithubCIEnvironment())
            return;
        const [email, name] = yield Promise.all([
            git.raw(['config', '--get', 'user.email']),
            git.raw(['config', '--get', 'user.name']),
        ]);
        const promises = [];
        if (!email) {
            promises.push(git.addConfig('user.name', 'dripip'));
        }
        if (!name) {
            promises.push(git.addConfig('user.email', 'dripip@prisma.io'));
        }
        yield Promise.all(promises);
    });
}
