"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const npm_auth_1 = require("../lib/npm-auth");
const publish_package_1 = require("../lib/publish-package");
const context_1 = require("../utils/context");
const context_checkers_1 = require("../utils/context-checkers");
const contrext_guard_1 = require("../utils/contrext-guard");
const output_1 = require("../utils/output");
const pr_release_1 = require("../utils/pr-release");
function runPullRequestRelease(options) {
    var e_1, _a;
    var _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        const cwd = (_b = options.cwd) !== null && _b !== void 0 ? _b : process.cwd();
        const readFromCIEnvironment = (_c = options.readFromCIEnvironment) !== null && _c !== void 0 ? _c : true;
        const context = yield context_1.getContext({ cwd, readFromCIEnvironment });
        const report = contrext_guard_1.check({ context })
            .errorUnless(context_checkers_1.npmAuthSetup())
            .errorUnless(branchHasOpenPR())
            // todo only we if can figoure the commits since last pr release
            // .must(haveMeaningfulCommitsInTheSeries())
            .run();
        // if we have preflight errors then we won't be able to calculate the
        // release info. But if doing a dry-run we want to be consistent with
        // other commands that embed report into returned data rather than
        // throwing an error.
        if (report.errors.length && options.dryRun) {
            return output_1.createDryRun({
                report: report,
                release: null,
            });
        }
        if (report.errors.length) {
            contrext_guard_1.guard({ context, report, json: options.json });
        }
        if (report.stops.length) {
            return output_1.createDidNotPublish({ reasons: report.stops });
        }
        const versionPrefix = `0.0.0-pr.${context.currentBranch.pr.number}.`;
        const versionBuildNum = pr_release_1.getNextPreReleaseBuildNum(context.package.name, versionPrefix);
        const version = `${versionPrefix}${versionBuildNum}.${context.series.current.sha.slice(0, 7)}`;
        const versionInfo = {
            major: 0,
            minor: 0,
            patch: 0,
            // prettier-ignore
            version: version,
            vprefix: false,
            preRelease: {
                identifier: 'pr',
                prNum: context.currentBranch.pr.number,
                shortSha: context.series.current.sha.slice(0, 7),
            },
        };
        // todo show publish plan in dryrun for other commands too
        const publishPlan = {
            release: {
                distTag: `pr.${context.currentBranch.pr.number}`,
                version: versionInfo.version,
            },
            options: {
                gitTag: 'none',
            },
        };
        if (options.dryRun) {
            return output_1.createDryRun({
                report: report,
                publishPlan: publishPlan,
            });
        }
        npm_auth_1.setupNPMAuthfileOnCI();
        try {
            for (var _d = __asyncValues(publish_package_1.publishPackage(publishPlan)), _e; _e = yield _d.next(), !_e.done;) {
                const progress = _e.value;
                // turn this func into async iterator and yield structured progress
                if (options.progress) {
                    console.log(progress);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d.return)) yield _a.call(_d);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return output_1.createDidPublish({ release: publishPlan.release });
    });
}
exports.runPullRequestRelease = runPullRequestRelease;
//
// Validators
//
function branchHasOpenPR() {
    return {
        code: 'pr_release_without_open_pr',
        summary: 'Pull-Request releases are only supported on branches with _open_ pull-requests',
        run(ctx) {
            return ctx.currentBranch.pr !== null;
        },
    };
}
