"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const changelog_1 = require("../lib/changelog");
const npm_auth_1 = require("../lib/npm-auth");
const publish_changelog_1 = require("../lib/publish-changelog");
const publish_package_1 = require("../lib/publish-package");
const context_1 = require("../utils/context");
const context_checkers_1 = require("../utils/context-checkers");
const contrext_guard_1 = require("../utils/contrext-guard");
const octokit_1 = require("../utils/octokit");
const output_1 = require("../utils/output");
const release_1 = require("../utils/release");
function runStableRelease(options) {
    var e_1, _a;
    var _b, _c, _d, _e, _f;
    return __awaiter(this, void 0, void 0, function* () {
        const cwd = (_b = options.cwd) !== null && _b !== void 0 ? _b : process.cwd();
        const readFromCIEnvironment = (_c = options.readFromCIEnvironment) !== null && _c !== void 0 ? _c : true;
        const context = yield context_1.getContext({
            cwd,
            readFromCIEnvironment,
            overrides: {
                trunk: (_e = (_d = options.overrides) === null || _d === void 0 ? void 0 : _d.trunk) !== null && _e !== void 0 ? _e : null,
            },
        });
        const report = contrext_guard_1.check({ context: context })
            .errorUnless(context_checkers_1.npmAuthSetup())
            .errorUnless(context_checkers_1.isTrunk())
            .errorUnless(context_checkers_1.branchSynced())
            .stopUnless(notAlreadyStableRelease())
            .stopUnless(haveMeaningfulCommitsInTheSeries())
            .run();
        const maybeRelease = release_1.getNextStable(context.series);
        const changelog = changelog_1.renderChangelog(context.series, { as: 'markdown' });
        if (options.dryRun) {
            return output_1.createDryRun({
                report,
                release: maybeRelease,
                commits: context.series.commitsInNextStable,
                changelog: changelog,
            });
        }
        if (report.errors.length) {
            contrext_guard_1.guard({ context, report, json: options.json });
        }
        if (report.stops.length) {
            return output_1.createDidNotPublish({ reasons: report.stops });
        }
        const release = maybeRelease; // now validated
        const publishPlan = {
            release: {
                version: release.version.version,
                distTag: 'latest',
                extraDistTags: ['next'],
            },
            options: {
                npm: ((_f = options.overrides) === null || _f === void 0 ? void 0 : _f.skipNpm) !== true,
            },
        };
        npm_auth_1.setupNPMAuthfileOnCI();
        try {
            for (var _g = __asyncValues(publish_package_1.publishPackage(publishPlan)), _h; _h = yield _g.next(), !_h.done;) {
                const progress = _h.value;
                if (!options.json) {
                    console.log(progress);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_h && !_h.done && (_a = _g.return)) yield _a.call(_g);
            }
            finally { if (e_1) throw e_1.error; }
        }
        const releaseInfo = Object.assign(Object.assign({}, release), { head: {
                sha: context.series.current.sha,
            } });
        if (options.changelog && !options.dryRun) {
            yield publish_changelog_1.publishChangelog({
                octokit: octokit_1.octokit,
                release: releaseInfo,
                repo: context.githubRepo,
                body: changelog,
            });
        }
        return output_1.createDidPublish({ release: Object.assign({ notes: changelog }, publishPlan.release) });
    });
}
exports.runStableRelease = runStableRelease;
//
// Validators
//
function haveMeaningfulCommitsInTheSeries() {
    return {
        code: 'series_only_has_meaningless_commits',
        summary: 'A stable release must have at least one semantic commit',
        run(ctx) {
            const release = release_1.getNextStable(ctx.series);
            return release !== 'no_meaningful_change';
            // todo
            // hint:   //             'All commits are either meta or not conforming to conventional commit. No release will be made.',
        },
    };
}
function notAlreadyStableRelease() {
    return {
        code: 'commit_already_has_stable_release',
        summary: 'A stable release requires the commit to have no existing stable release',
        run(ctx) {
            return ctx.series.current.releases.stable === null;
        },
    };
}
