"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Git = __importStar(require("../lib/git"));
const git2_1 = require("../lib/git2");
const github_ci_environment_1 = require("../lib/github-ci-environment");
const PackageJson = __importStar(require("../lib/package-json"));
const octokit_1 = require("./octokit");
const Rel = __importStar(require("./release"));
function getContext(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const locationContext = yield getLocationContext({ octokit: octokit_1.octokit, options });
        const series = yield Rel.getCurrentSeries({ cwd: options.cwd });
        return Object.assign({ series }, locationContext);
    });
}
exports.getContext = getContext;
/**
 * Get location-oriented contextual information. Does not consider the release
 * series but things like current branch, repo, pr etc.
 */
function getLocationContext({ octokit, options, }) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const git = git2_1.createGit({ cwd: options === null || options === void 0 ? void 0 : options.cwd });
        const readFromCIEnvironment = options === null || options === void 0 ? void 0 : options.readFromCIEnvironment;
        let githubCIEnvironment = null;
        if (readFromCIEnvironment) {
            githubCIEnvironment = github_ci_environment_1.parseGithubCIEnvironment();
        }
        // Get repo info
        let repoInfo;
        repoInfo = githubCIEnvironment === null || githubCIEnvironment === void 0 ? void 0 : githubCIEnvironment.parsed.repo;
        if (!repoInfo) {
            repoInfo = yield Git.parseGithubRepoInfoFromGitConfig();
        }
        // Get which branch is trunk, overridable
        let trunkBranch;
        if ((_a = options === null || options === void 0 ? void 0 : options.overrides) === null || _a === void 0 ? void 0 : _a.trunk) {
            trunkBranch = options.overrides.trunk;
        }
        else {
            let githubRepo;
            try {
                githubRepo = yield octokit.repos.get({
                    owner: repoInfo.owner,
                    repo: repoInfo.name,
                });
            }
            catch (e) {
                throw new Error(`Failed to fetch repo info from ${repoInfo.owner}/${repoInfo.name} in order to get the default branch.\n\n${e}`);
            }
            trunkBranch = githubRepo.data.default_branch;
        }
        // Get the branch
        // const branchesSummary = await git.branch({})
        let currentBranchName = yield git.getCurrentBranchName();
        if (!currentBranchName && githubCIEnvironment && githubCIEnvironment.parsed.branchName) {
            currentBranchName = githubCIEnvironment.parsed.branchName;
        }
        if (!currentBranchName) {
            throw new Error('Could not get current branch name');
        }
        // Get the pr
        let pr = null;
        if (githubCIEnvironment && githubCIEnvironment.parsed.prNum) {
            pr = {
                number: githubCIEnvironment.parsed.prNum,
            };
        }
        else {
            const head = `${repoInfo.owner}:${currentBranchName}`;
            const owner = repoInfo.owner;
            const repo = repoInfo.name;
            const state = 'open';
            let maybePR;
            try {
                maybePR = (yield octokit.pulls.list({ owner, repo, head, state })).data[0];
            }
            catch (e) {
                throw new Error(`Failed to fetch ${state} pull requests from ${owner}/${name} for head ${head} in order to find out if this branch has an open pull-request.\n\n${e}`);
            }
            if (maybePR) {
                pr = {
                    number: maybePR.number,
                };
            }
        }
        // get the branch sync status
        const syncStatus = yield git.checkSyncStatus({
            branchName: currentBranchName,
        });
        // get package info
        const packageJson = yield PackageJson.getPackageJson();
        return {
            package: {
                name: packageJson.name,
            },
            githubRepo: {
                owner: repoInfo.owner,
                name: repoInfo.name,
                trunkBranch: trunkBranch,
            },
            currentBranch: {
                name: currentBranchName,
                isTrunk: currentBranchName === trunkBranch,
                pr: pr,
                syncStatus: syncStatus,
            },
        };
    });
}
exports.getLocationContext = getLocationContext;
